package style

import (
	"math"
	"regexp"
	"strconv"
	"strings"
)

var hex256 = map[string]string{
	"#000000": "color0",
	"#800000": "color1",
	"#008000": "color2",
	"#808000": "color3",
	"#000080": "color4",
	"#800080": "color5",
	"#008080": "color6",
	"#c0c0c0": "color7",
	"#808080": "color8",
	"#ff0000": "color9",
	"#00ff00": "color10",
	"#ffff00": "color11",
	"#0000ff": "color12",
	"#ff00ff": "color13",
	"#00ffff": "color14",
	"#ffffff": "color15",
	//"#000000": "color16",
	"#00005f": "color17",
	"#000087": "color18",
	"#0000af": "color19",
	"#0000d7": "color20",
	//"#0000ff": "color21",
	"#005f00": "color22",
	"#005f5f": "color23",
	"#005f87": "color24",
	"#005faf": "color25",
	"#005fd7": "color26",
	"#005fff": "color27",
	"#008700": "color28",
	"#00875f": "color29",
	"#008787": "color30",
	"#0087af": "color31",
	"#0087d7": "color32",
	"#0087ff": "color33",
	"#00af00": "color34",
	"#00af5f": "color35",
	"#00af87": "color36",
	"#00afaf": "color37",
	"#00afd7": "color38",
	"#00afff": "color39",
	"#00d700": "color40",
	"#00d75f": "color41",
	"#00d787": "color42",
	"#00d7af": "color43",
	"#00d7d7": "color44",
	"#00d7ff": "color45",
	//"#00ff00": "color46",
	"#00ff5f": "color47",
	"#00ff87": "color48",
	"#00ffaf": "color49",
	"#00ffd7": "color50",
	//"#00ffff": "color51",
	"#5f0000": "color52",
	"#5f005f": "color53",
	"#5f0087": "color54",
	"#5f00af": "color55",
	"#5f00d7": "color56",
	"#5f00ff": "color57",
	"#5f5f00": "color58",
	"#5f5f5f": "color59",
	"#5f5f87": "color60",
	"#5f5faf": "color61",
	"#5f5fd7": "color62",
	"#5f5fff": "color63",
	"#5f8700": "color64",
	"#5f875f": "color65",
	"#5f8787": "color66",
	"#5f87af": "color67",
	"#5f87d7": "color68",
	"#5f87ff": "color69",
	"#5faf00": "color70",
	"#5faf5f": "color71",
	"#5faf87": "color72",
	"#5fafaf": "color73",
	"#5fafd7": "color74",
	"#5fafff": "color75",
	"#5fd700": "color76",
	"#5fd75f": "color77",
	"#5fd787": "color78",
	"#5fd7af": "color79",
	"#5fd7d7": "color80",
	"#5fd7ff": "color81",
	"#5fff00": "color82",
	"#5fff5f": "color83",
	"#5fff87": "color84",
	"#5fffaf": "color85",
	"#5fffd7": "color86",
	"#5fffff": "color87",
	"#870000": "color88",
	"#87005f": "color89",
	"#870087": "color90",
	"#8700af": "color91",
	"#8700d7": "color92",
	"#8700ff": "color93",
	"#875f00": "color94",
	"#875f5f": "color95",
	"#875f87": "color96",
	"#875faf": "color97",
	"#875fd7": "color98",
	"#875fff": "color99",
	"#878700": "color100",
	"#87875f": "color101",
	"#878787": "color102",
	"#8787af": "color103",
	"#8787d7": "color104",
	"#8787ff": "color105",
	"#87af00": "color106",
	"#87af5f": "color107",
	"#87af87": "color108",
	"#87afaf": "color109",
	"#87afd7": "color110",
	"#87afff": "color111",
	"#87d700": "color112",
	"#87d75f": "color113",
	"#87d787": "color114",
	"#87d7af": "color115",
	"#87d7d7": "color116",
	"#87d7ff": "color117",
	"#87ff00": "color118",
	"#87ff5f": "color119",
	"#87ff87": "color120",
	"#87ffaf": "color121",
	"#87ffd7": "color122",
	"#87ffff": "color123",
	"#af0000": "color124",
	"#af005f": "color125",
	"#af0087": "color126",
	"#af00af": "color127",
	"#af00d7": "color128",
	"#af00ff": "color129",
	"#af5f00": "color130",
	"#af5f5f": "color131",
	"#af5f87": "color132",
	"#af5faf": "color133",
	"#af5fd7": "color134",
	"#af5fff": "color135",
	"#af8700": "color136",
	"#af875f": "color137",
	"#af8787": "color138",
	"#af87af": "color139",
	"#af87d7": "color140",
	"#af87ff": "color141",
	"#afaf00": "color142",
	"#afaf5f": "color143",
	"#afaf87": "color144",
	"#afafaf": "color145",
	"#afafd7": "color146",
	"#afafff": "color147",
	"#afd700": "color148",
	"#afd75f": "color149",
	"#afd787": "color150",
	"#afd7af": "color151",
	"#afd7d7": "color152",
	"#afd7ff": "color153",
	"#afff00": "color154",
	"#afff5f": "color155",
	"#afff87": "color156",
	"#afffaf": "color157",
	"#afffd7": "color158",
	"#afffff": "color159",
	"#d70000": "color160",
	"#d7005f": "color161",
	"#d70087": "color162",
	"#d700af": "color163",
	"#d700d7": "color164",
	"#d700ff": "color165",
	"#d75f00": "color166",
	"#d75f5f": "color167",
	"#d75f87": "color168",
	"#d75faf": "color169",
	"#d75fd7": "color170",
	"#d75fff": "color171",
	"#d78700": "color172",
	"#d7875f": "color173",
	"#d78787": "color174",
	"#d787af": "color175",
	"#d787d7": "color176",
	"#d787ff": "color177",
	"#d7af00": "color178",
	"#d7af5f": "color179",
	"#d7af87": "color180",
	"#d7afaf": "color181",
	"#d7afd7": "color182",
	"#d7afff": "color183",
	"#d7d700": "color184",
	"#d7d75f": "color185",
	"#d7d787": "color186",
	"#d7d7af": "color187",
	"#d7d7d7": "color188",
	"#d7d7ff": "color189",
	"#d7ff00": "color190",
	"#d7ff5f": "color191",
	"#d7ff87": "color192",
	"#d7ffaf": "color193",
	"#d7ffd7": "color194",
	"#d7ffff": "color195",
	//"#ff0000": "color196",
	"#ff005f": "color197",
	"#ff0087": "color198",
	"#ff00af": "color199",
	"#ff00d7": "color200",
	//"#ff00ff": "color201",
	"#ff5f00": "color202",
	"#ff5f5f": "color203",
	"#ff5f87": "color204",
	"#ff5faf": "color205",
	"#ff5fd7": "color206",
	"#ff5fff": "color207",
	"#ff8700": "color208",
	"#ff875f": "color209",
	"#ff8787": "color210",
	"#ff87af": "color211",
	"#ff87d7": "color212",
	"#ff87ff": "color213",
	"#ffaf00": "color214",
	"#ffaf5f": "color215",
	"#ffaf87": "color216",
	"#ffafaf": "color217",
	"#ffafd7": "color218",
	"#ffafff": "color219",
	"#ffd700": "color220",
	"#ffd75f": "color221",
	"#ffd787": "color222",
	"#ffd7af": "color223",
	"#ffd7d7": "color224",
	"#ffd7ff": "color225",
	//"#ffff00": "color226",
	"#ffff5f": "color227",
	"#ffff87": "color228",
	"#ffffaf": "color229",
	"#ffffd7": "color230",
	//"#ffffff": "color231",
	"#080808": "color232",
	"#121212": "color233",
	"#1c1c1c": "color234",
	"#262626": "color235",
	"#303030": "color236",
	"#3a3a3a": "color237",
	"#444444": "color238",
	"#4e4e4e": "color239",
	"#585858": "color240",
	"#626262": "color241",
	"#6c6c6c": "color242",
	"#767676": "color243",
	//"#808080": "color244",
	"#8a8a8a": "color245",
	"#949494": "color246",
	"#9e9e9e": "color247",
	"#a8a8a8": "color248",
	"#b2b2b2": "color249",
	"#bcbcbc": "color250",
	"#c6c6c6": "color251",
	"#d0d0d0": "color252",
	"#dadada": "color253",
	"#e4e4e4": "color254",
	"#eeeeee": "color255",
}

// Hex256 returns nearest 256 color style for given hex
func Hex256(s string) string {
	if !strings.HasPrefix(s, "#") {
		s = "#" + s
	}

	if _style, ok := hex256[strings.ToLower(s)]; ok {
		return _style
	} else if r := regexp.MustCompile(`^#(?P<r>..)(?P<g>..)(?P<b>..)$`); r.MatchString(s) {
		matches := r.FindStringSubmatch(s)
		r, errR := strconv.ParseUint(matches[1], 16, 32)
		g, errG := strconv.ParseUint(matches[2], 16, 32)
		b, errB := strconv.ParseUint(matches[3], 16, 32)
		if errR == nil && errG == nil && errB == nil {
			return Color256(rgbToXterm(int(r), int(g), int(b)))
		}
	}
	return Default
}

var n []int

func init() {
	n = make([]int, 0)
	for i, v := range []int{47, 68, 40, 40, 40, 21} {
		for j := 0; j < v; j++ {
			n = append(n, i)
		}
	}
}

// rgbToXterm returns nearest 256 color for given rgb
// source: https://stackoverflow.com/a/62219320
func rgbToXterm(r, g, b int) int {
	mx := math.Max(math.Max(float64(r), float64(g)), float64(b))
	mn := math.Min(math.Min(float64(r), float64(g)), float64(b))

	if (mx-mn)*(mx+mn) <= 6250 {
		c := 24 - (252-((r+g+b)%3))%10
		if 0 <= c && c <= 23 {
			return 232 + c
		}
	}
	return 16 + 36*n[r] + 6*n[g] + n[b]
}
